@@ -7,6 +7,7 @@ use JSON;
 use Test::APIcast -Base;
 use File::Copy "move";
 use File::Temp qw/ tempfile /;
+use File::Slurp qw(read_file);
 
 BEGIN {
     $ENV{APICAST_OPENRESTY_BINARY} = $ENV{TEST_NGINX_BINARY};
@@ -15,6 +16,7 @@ BEGIN {
 our $ApicastBinary = $ENV{TEST_NGINX_APICAST_BINARY} || 'bin/apicast';
 
 our %EnvToNginx = ();
+our %ResetEnv = ();
 
 sub env_to_apicast (@) {
     my %env = (@_);
@@ -105,6 +107,7 @@ _EOC_
 my $write_nginx_config = sub {
     my $block = shift;
 
+    my $FilterHttpConfig = $Test::Nginx::Util::FilterHttpConfig;
     my $ConfFile = $Test::Nginx::Util::ConfFile;
     my $Workers = $Test::Nginx::Util::Workers;
     my $MasterProcessEnabled = $Test::Nginx::Util::MasterProcessEnabled;
@@ -151,14 +154,22 @@ my $write_nginx_config = sub {
     }
 
     my %env = (%EnvToNginx, $block->env);
-    my @env_list = ();
+
+    # reset ENV to memorized state
+    for my $key (keys %ResetEnv) {
+        $ENV{$key} = $ResetEnv{$key};
+        delete $ResetEnv{$key};
+    }
 
     for my $key (keys %env) {
-        push @env_list, "$key='$env{$key}'";
+        # memorize ENV before changing it
+        $ResetEnv{$key} = $ENV{$key};
+        # change ENV to state desired by the test
+        $ENV{$key} = $env{$key};
     }
 
     my ($env, $env_file) = tempfile();
-    my $apicast_cmd = "${\(join(' ', @env_list))} APICAST_CONFIGURATION_LOADER='test' $apicast_cli start --test --environment $env_file";
+    my $apicast_cmd = "APICAST_CONFIGURATION_LOADER='test' $apicast_cli start --test --environment $env_file";
 
     if (defined $configuration_file) {
         $apicast_cmd .= " --configuration $configuration_file"
@@ -188,7 +199,7 @@ return {
     },
     env = {
         THREESCALE_CONFIG_FILE = [[$configuration_file]],
-        APICAST_CONFIGURATION_LOADER = 'boot', ${\(join(', ', @env_list))}
+        APICAST_CONFIGURATION_LOADER = 'boot'
     },
     server_name = {
         management = $management_server_name
@@ -206,7 +217,16 @@ _EOC_
     my $apicast = `${apicast_cmd} 2>&1`;
     if ($apicast =~ /configuration file (?<file>.+?) test is successful/)
     {
-        move($+{file}, $ConfFile);
+        open(my $fh, '+>', $ConfFile) or die "cannot open $ConfFile: $!";
+
+        my $nginx_config = read_file($+{file});
+
+        if ($FilterHttpConfig) {
+            $nginx_config = $FilterHttpConfig->($nginx_config);
+        }
+
+        print { $fh } $nginx_config;
+        close($fh);
     } else {
         warn "Missing config file: $Test::Nginx::Util::ConfFile";
         warn $apicast;
